% script file illustrate_gradient.m
% illustrate the gradient methods for different
% matrices in two dimensions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc; clear all; close all;

%A = [-5 1; 1 -20];
A = [5 1; 1 20];


b = [0; 0];

% visualise the functional Phi
x = linspace(-2.5,2.5,500)';
y = linspace(-2.5,2.5,500)';
[X,Y] = ndgrid(x,y);


Phi = min_functional(A,b,X,Y);


figure(1);
P=surf(X,Y,Phi);
shading flat;
%set(P,'facecolor',[0.8 0.8 0.8]);
hold on

% mark the solution
exact = A\b;

plot3(exact(1),exact(2),min_functional(A,b,exact(1),exact(2)),'r*')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% gradient method
u0 = [1;1];

maxit = 10;

figure(20)
P=surf(X,Y,Phi);
hold on; shading flat
PP = plot3(u0(1),u0(2),min_functional(A,b,u0(1),u0(2)),'ko');
set(PP,'markerfacecolor','g');

figure(2);
hold on
P=plot(exact(1),exact(2),'go');
set(P,'markerfacecolor','g');

plot(u0(1),u0(2),'b*');


t = linspace(0,1,50)'; % for visualisation

for ctr=1:maxit
  % u update
  ru = b - A*u0;
  alphau = (ru'*ru)/(ru'*(A*ru));
  
  u = u0+alphau*ru;
      
  if 1
    figure(20);
    % plot a parameterised curve on the surface
    xx = (u(1)-u0(1))*t + u0(1);
    yy = (u(2)-u0(2))*t + u0(2);
    phi = min_functional(A,b,xx,yy);
    P = plot3(xx,yy,phi,'k');
    set(P,'linewidth',2);
    PP = plot3(xx(end),yy(end),phi(end),'ko');
    set(PP,'markerfacecolor','g');
  end
  
  if 1
    figure(2)  
    L=line([u0(1); u(1)],[u0(2); u(2)]);
    phi = min_functional(A,b,u0(1),u0(2));
    [C,H]=contour(X,Y,Phi,[phi phi]);
    set(H,'linecolor','r');  
  end
    
  u0 = u;
  
end

figure(2)
xlabel('x');
ylabel('y');
axis equal
title('Gradient method');  


figure(20)
xlabel('x');
ylabel('y');
axis square
title('Gradient method');  
colormap(spring);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Richardson method
u0 = [1;1];
w = 0.1;

figure(30)
P=surf(X,Y,Phi);
hold on; shading flat


figure(3);
hold on
P=plot(exact(1),exact(2),'go');
set(P,'markerfacecolor','g');

plot(u0(1),u0(2),'b*');


for ctr=1:maxit
  ru = b - A*u0; 
  
  u = u0+w*ru;
  
  if 1
    figure(30);
    % plot a parameterised curve on the surface
    xx = (u(1)-u0(1))*t + u0(1);
    yy = (u(2)-u0(2))*t + u0(2);
    phi = min_functional(A,b,xx,yy);
    P = plot3(xx,yy,phi,'k');
    set(P,'linewidth',2);
    PP = plot3(xx(end),yy(end),phi(end),'ko');
    set(PP,'markerfacecolor','g');
  end
    
  if 1  
    figure(3)
    L=line([u0(1); u(1)],[u0(2); u(2)]);
    %set(L,'linewidth',2)
    phi = min_functional(A,b,u0(1),u0(2));
    [C,H]=contour(X,Y,Phi,[phi phi]);
    set(H,'linecolor','r');  
  end
    
  u0 = u;  
end

figure(3)
xlabel('x');
ylabel('y');
axis equal
title('Richardson method');  


figure(30)
xlabel('x');
ylabel('y');
axis square
title('Richardson method');  
colormap(spring)


